/*
    Qalculate

    Copyright (C) 2003-2005  Hanna Knutsson (hanna.knutsson@protonmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

#ifndef QALCULATE_H
#define QALCULATE_H

#include <libqalculate/includes.h>
#include <libqalculate/util.h>
#include <libqalculate/Calculator.h>
#include <libqalculate/MathStructure.h>
#include <libqalculate/Number.h>
#include <libqalculate/Prefix.h>
#include <libqalculate/ExpressionItem.h>
#include <libqalculate/Variable.h>
#include <libqalculate/Function.h>
#include <libqalculate/DataSet.h>
#include <libqalculate/Unit.h>
#include <libqalculate/BuiltinFunctions.h>

#endif
